function [Xvec] = maxcare (Avec,Bvec,Qvec,Lvec,Rvec,P,options,v); 
%MAXCARE Finds a maximal solution for a set of Coupled Algebraic
%	Riccati Equations (CARE).
%
%	Usage:
%		Xvec = maxcare(Avec,Bvec,Qvec,Lvec,Rvec,P,OPTIONS,V);
%
%	where the arguments (.)vec are vectorized with the use of the
%	function STACK (e.g. Avec=stack(A1,A2,A3); ). The argument P
%	is the Transition Probability Matrix and the argument OPTIONS
%	contains the options for the optimization algorithm (type HELP
%	MINCX for more information). The argument V (meaning verbose
%	mode) controls the output of some additional information about
%	the construction of the optimization problem.
%	The maximal solution, if it exists, can be obtained with the
%	function UNSTACK (e.g. [X1,X2,X3]=unstack(Xvec); ).
%
%	Example (for two operation modes):
%
%		A1=...; A2=...; etc.
%		Avec=stack(A1,A2); Bvec=stack(B1,B2);
%		Qvec=stack(Q1,Q2);
%		Lvec=stack(L1,L2); Rvec=stack(R1,R2);
%		P=...; options=...;
%		Xvec = maxcare(...);
%		[X1,X2]=unstack(Xvec);
%

%	RP Marques
%	Version 1.1	(09/10/96)

% Parameter checking
if ~( isnan(Avec(1)) & isnan(Bvec(1)) & isnan(Qvec(1)) & isnan(Lvec(1)) & isnan(Rvec(1)) )
	error('Invalid argument(s)')
end; 
N = Avec(2);
if (Bvec(2)~=N) | (Qvec(2)~=N) | (Lvec(2)~=N) | (Rvec(2)~=N)
	error('Invalid number of operation modes for some argument(s)')
end;
if nargin<8, v=0; end;
if nargin<7, options=zeros(1,5); end;

% Problem dimensions
for i=1:N
	n(i)=Avec(2*i+1);
end;

% Argument unpacking
for i=1:N
	istr=int2str(i);
	eval(['A',istr,'=unstack(Avec,',istr,');']);
	eval(['B',istr,'=unstack(Bvec,',istr,');']);
	eval(['Q',istr,'=unstack(Qvec,',istr,');']);
	eval(['L',istr,'=unstack(Lvec,',istr,');']);
	eval(['R',istr,'=unstack(Rvec,',istr,');']);
end;

% Problem formulation

%%% 1. initialization  
setlmis([]);
if v, fprintf('\n\nProblem construction:\n\n\n'); end;

%%% 2. Variable declaration  
if v, fprintf('1) Variable declaration\n\n'); end;     
for i=1:N
	eval(['X',int2str(i),'=lmivar(1,[',int2str(n(i)),' 1]);'])
	if v, disp(['X',int2str(i),'=lmivar(1,[',int2str(n(i)),' 1]);']); end;
end;


%%% 3. Restriction declaration
if v, fprintf('\n2) Inequalities\n\n'); end;
for i=1:N
	istr=int2str(i);
	eqstr1=int2str(2*(i-1)+1); eqstr2=int2str(2*i);

	% Inequality #1
	eval(['lmiterm([-',eqstr1,' 1 1 X',istr,'],1,-1);'])
	if v, disp(['lmiterm([-',eqstr1,' 1 1 X',istr,'],1,-1);']); end;
	for j=1:N
		jstr=int2str(j);
		eval(['lmiterm([-',eqstr1,' 1 1 X',jstr,'],P(',istr,',',jstr,')*A',istr,''',A',istr,');'])
		if v, disp(['lmiterm([-',eqstr1,' 1 1 X',jstr,'],P(',istr,',',jstr,')*A',istr,''',A',istr,');']); end;
	end;
	eval(['lmiterm([-',eqstr1,' 1 1 0],Q',istr,');'])
	if v, disp(['lmiterm([-',eqstr1,' 1 1 0],Q',istr,');']); end;

	for j=1:N
		jstr=int2str(j);
		eval(['lmiterm([-',eqstr1,' 2 1 X',jstr,'],P(',istr,',',jstr,')*B',istr,''',A',istr,');'])
		if v, disp(['lmiterm([-',eqstr1,' 2 1 X',jstr,'],P(',istr,',',jstr,')*B',istr,''',A',istr,');']); end;
	end;
	eval(['lmiterm([-',eqstr1,' 2 1 0],L',istr,''');'])
	if v, disp(['lmiterm([-',eqstr1,' 2 1 0],L',istr,''');']); end;

	for j=1:N
		jstr=int2str(j);
		eval(['lmiterm([-',eqstr1,' 2 2 X',jstr,'],P(',istr,',',jstr,')*B',istr,''',B',istr,');'])
		if v, disp(['lmiterm([-',eqstr1,' 2 2 X',jstr,'],P(',istr,',',jstr,')*B',istr,''',B',istr,');']); end;
	end;
	eval(['lmiterm([-',eqstr1,' 2 2 0],R',istr,');'])
	if v, disp(['lmiterm([-',eqstr1,' 2 2 0],R',istr,');']); end;

	% Inequality #2
	for j=1:N
		jstr=int2str(j);
		eval(['lmiterm([-',eqstr2,' 1 1 X',jstr,'],P(',istr,',',jstr,')*B',istr,''',B',istr,');'])
		if v, disp(['lmiterm([-',eqstr2,' 1 1 X',jstr,'],P(',istr,',',jstr,')*B',istr,''',B',istr,');']); end;
	end;
	eval(['lmiterm([-',eqstr2,' 1 1 0],R',istr,');'])
	if v, disp(['lmiterm([-',eqstr2,' 1 1 0],R',istr,');']); end;
end;

%%% 4. Conclusion
sistema=getlmis;

% Objective function declaration
% (see page 8-30)

nsis=decnbr(sistema); c=zeros(nsis,1);

if v, fprintf('\n3) Calculating Objective Function.\n\n'); end;
Xstr1=[]; Xstr2=[]; Xstr3=[];
for i=1:N,
	istr=int2str(i);
	aux1=[',X',istr]; aux2=[',X',istr,'j']; aux3=['+X',istr,'j'];
	Xstr1=[Xstr1,aux1]; Xstr2=[Xstr2,aux2]; Xstr3=[Xstr3,aux3];
end;
Xstr1(1)=[]; Xstr2(1)=[]; Xstr3(1)=[];

for j=1:nsis,
	eval(['[',Xstr2,']=defcx(sistema,j,',Xstr1,');']);
	eval(['c(j)=-trace(',Xstr3,');']);
end;

% Optimization
if v, fprintf('\n4) Solving optimization problem.\n\n'); end;
[copt,xopt]=mincx(sistema,c,options,[],[]);

% Output
Xstr4=[];
for i=1:N
	istr=int2str(i); aux4=[',X',istr,'opt'];
	eval(['X',istr,'opt=dec2mat(sistema,xopt,X',istr,');']);
	Xstr4=[Xstr4,aux4];
end;
Xstr4(1)=[];
eval(['Xvec=stack(',Xstr4,');']);
