function [Zvec,Xvec,Wvec,C] = mcl2sim(Avec,Bvec,Fvec,Dvec,Mvec,Jvec,P,x0,th0,n,nsim,wopt,s);

%MCL2SIM Monte Carlo simulation of the L2-disturbed system
%
%	Usage:
%		
%	[Zvec,Xvec,Wvec,C] = mcl2sim(Avec,Bvec,Fvec,Dvec,Mvec,Jvec,P,X0,TH0,N,NSIM,WOPT,S);
%	or
%	Zvec=mcl2sim(...); [Zvec,Xvec]=mcl2sim(...); etc.
%
%	Zvec,Xvec,Wvec contain the history of the output,state and disturbance
%	for each simulation (unpack using UNSTACK) and C contains the realizations
%	of the Markov chains.
%
%	Gvec,...,Rvec contain the system and cost descriptions, P is the
%	probability transition matrix, X0 and TH0 are the initial conditions
%	for the system and for the Markov chain. N is the number of steps
%	for each simulation and NSIM is the number of simulations. S (optional)
%	is the seed for the random number generator.

%	Version 1.0
%	RP Marques	(13/03/97)

% Argument checking
if size(P,1)==size(P,2),
	N=size(P,1);
else
	error('Invalid Transition Probability Matrix');
end;

if th0 > N | th0 < 1
	error('Invalid initial state');
end;

% Initialization
if nargin >= 1
	dimZ=Mvec(3); Zvec=zeros(2+nsim*2+nsim*dimZ*n,1);
	Zvec(1)=nan; Zvec(2)=nsim;
	for i=1:nsim,
	   Zvec(2*i+1:2*i+2)=[dimZ;n];
	end;
end;
if nargin >= 2
	dimX=Avec(3); Xvec=zeros(2+nsim*2+nsim*dimX*n,1);
	Xvec(1)=nan; Xvec(2)=nsim;
	for i=1:nsim,
	   Xvec(2*i+1:2*i+2)=[dimX;n];
	end;
end;
if nargin >= 3
	dimW=Dvec(4); Wvec=zeros(2+nsim*2+nsim*dimW*n,1);
	Wvec(1)=nan; Wvec(2)=nsim;
	for i=1:nsim,
	   Wvec(2*i+1:2*i+2)=[dimW;n];
	end;
end;

% Seed generation
if length(wopt)==3,
	rand('seed',wopt(3));
elseif length(wopt)==2,
	rand('seed',sum(100*clock));
end;
saux1=floor(1e7*rand(nsim,1));

if nargin==12,
	s = sum(100*clock);
elseif s==[],
	s = sum(100*clock);
end;
rand('seed',s);
saux2=floor(1e7*rand(nsim,1));


% Simulation
fprintf('\n\nWorking...\n\n');
fprintf('000      50      100\n');
fprintf('+--------++--------+\n');

istar = 1;

for i=1:nsim,
	% Simulate
	optaux = [wopt(1);wopt(2);saux1(i)];
	[Zaux,Xaux,Waux,C(:,i)]=l2sim(Avec,Bvec,Fvec,Dvec,Mvec,Jvec,P,x0,th0,n,optaux,saux2(i));

	% Put .aux into .vec
	Zvec(2+nsim*2+(i-1)*dimZ*n+1:2+nsim*2+i*dimZ*n) = Zaux(:);
	if nargin >= 2
	   Xvec(2+nsim*2+(i-1)*dimX*n+1:2+nsim*2+i*dimX*n) = Xaux(:);
	end;
	if nargin >=3
	   Wvec(2+nsim*2+(i-1)*dimW*n+1:2+nsim*2+i*dimW*n) = Waux(:);
	end;	   

	% Tell the operator about computation progress
	if floor(i*20/nsim) >= istar;
	   fprintf('*');
	   istar = istar+1;
	end;
end;

% Finalization
for i=istar:20;
	fprintf('*');
end;
fprintf('\n\n');
end;