function [Fvec,nuopt,Xvec] = solvhin2(Avec,Bvec,Dvec,Mvec,Jvec,P,nu_0,tol,opt1,opt2,v)

%SOLVHIN2 Finds the optimal solution for the Hinf control problem.
%
%	Usage:
%		[Fvec,NUOPT,Xvec] = solvhin2(Avec,Bvec,Dvec,Mvec,Jvec,P,NU_0TOL,OP1,OP2,V);
%		or
%		[Fvec,NUOPT,Xvec] = solvhin2(Avec,Bvec,D,Mvec,Jvec,P,NU_0,TOL);
%		etc.
%
%	SOLVHIN2 uses routine HINFSOPT to produce an exact solution to the optimal
%	Hinf control problem iteratively via a bissection method.
%
%	Fvec contains the optimal controller, NUOPT is an upper bound for the Hinf norm
%	and Xvec is the solution for the CARE associated to the Hinf control problem.
%
%	Avec,...,Jvec contain the system description, P is the transition probability
%	matrix, NU_0 is a initial guess for the Hinf norm, TOL is the absolute tolerance
%	for the solution, OP1 and OP2 are options for the HINFSOPT routine (type HELP
%	HINFSOPT for more information).
%
%	NU_0 must be a solution to the Hinf suboptimal problem. If it's not, this routine
%	will increase its value until the suboptimal problem is satisfied.

%	Version 1.0
%	RP Marques	(11/03/97)

% Argument checking
if     nargin == 10,
	v = 0;
elseif nargin == 9,
	opt2 = [0 200 0 0 1]; v = 0;
elseif nargin == 8,
	opt1 = []; opt2 = [0 200 0 0 1]; v = 0;
elseif nargin <= 7,
	error('Incorrect number of arguments');
end;

if nu_0 == 0, error('Invalid initial condition for nu'); end;

%%% 1). Initial solution
if v
	fprintf('\n\n----------------------------\n');
	fprintf(    'Optimal Hinf control Problem\n');
	fprintf(    '----------------------------\n\n');
	fprintf(    '1. Initial condition\n');
	pause(0.2);
end;

[Fvec,Xvec,infodat] = hinfsopt(Avec,Bvec,Dvec,Mvec,Jvec,P,nu_0,opt1,opt2);
if infodat(1)<0 | infodat(2)==1 | infodat(3)==1
	fprintf('   Calculating initial solution (this may take long)\n');
	while infodat(1)<0 | infodat(2)==1 | infodat(3)==1
	   lo_nu = nu_0; nu_0 = 2*nu_0;
	   [Fvec,Xvec,infodat] = hinfsopt(Avec,Bvec,Dvec,Mvec,Jvec,P,nu_0,opt1,opt2);
	end;
else
	lo_nu = 0;
end;

up_nu = nu_0;

%%% 2). Iteration phase

n_it = ceil(log(tol/(up_nu-lo_nu))/log(0.5));

if v
	fprintf('\n2. Iteration phase\n');
	fprintf('   Desired tolerance will be achieved in %d steps\n\n',n_it);
	fprintf('--st.#---Dir---LowBound----UppBound----CurValue----Comments------------\n');
	pause(0.2);
end;

it = 1;

while (up_nu - lo_nu) >= tol
	
	nu_cur = lo_nu + (up_nu-lo_nu)/2;

	[Fvec,Xvec,infodat] = hinfsopt(Avec,Bvec,Dvec,Mvec,Jvec,P,nu_cur,opt1,opt2);

	if infodat(1)<0 | infodat(2)==1 | infodat(3)==1
	   % No convergence
	   if v
	      arrow = '>>';
	      if infodat(1)<0
	         diagstr = 'No convergence';
	      elseif infodat(2)==1 & infodat(3)==1
	         diagstr = 'Conditions (i) and (ii) failed';
	      elseif infodat(2)==1
	         diagstr = 'Condition (i) failed';
	      elseif infodat(3)==1
	         diagstr = 'Condition (ii) failed';
	      end;
	      fprintf('%6.0f\t %s    %1.3e  %1.3e  %1.3e  %s\n',it,arrow,lo_nu,up_nu,nu_cur,diagstr);
	      pause(0.2);
	   end;
	   lo_nu = nu_cur;
	else
	   % Convergence
	   if v
	      arrow = '<<';
	      diagstr = sprintf('Convergence in %d iterations',infodat(1));
	      fprintf('%6.0f\t %s    %1.3e  %1.3e  %1.3e  %s\n',it,arrow,lo_nu,up_nu,nu_cur,diagstr);
	      pause(0.2);
	   end;
	   up_nu = nu_cur;
	end;

	it = it+1;
end;	

%%% 3). Conclusion

nuopt = up_nu;
	