function [Fvec,nuopt,Xvec] = solvhinf(Avec,Bvec,D,Mvec,Jvec,P,tol,opt1,opt2,opt3,v)

%SOLVHINF Finds the optimal solution for the Hinf control problem.
%
%	Usage:
%		[Fvec,NUOPT,Xvec] = solvhinf(Avec,Bvec,D,Mvec,Jvec,P,TOL,OP1,OP2,OP3,V);
%		or
%		[Fvec,NUOPT,Xvec] = solvhinf(Avec,Bvec,D,Mvec,Jvec,P,TOL);
%		etc.
%
%	SOLVHINF combines routines HINFOPT and HINFSOPT to produce an exact solution
%	to the optimal Hinf control problem. HINFOPT produces a first approximation,
%	refined iteratively with HINFSOPT.
%
%	Fvec contains the optimal controller, NUOPT is an upper bound for the Hinf norm
%	and Xvec is the solution for the CARE associated to the Hinf control problem.
%
%	Avec,...,Jvec contain the system description (note that D must be a single
%	matrix), P is the transition probability matrix, TOL is the absolute tolerance
%	for the solution, OP1 and OP2 are options for the HINFSOPT routine (type HELP
%	HINFSOPT for more information) and OP3 is the option vector for the HINFOPT
%	routine (type HELP MINCX for more information).

%	Version 1.0
%	RP Marques	(11/03/97)

% Argument checking
if     nargin == 10,
	v = 0;
elseif nargin == 9,
	opt3 = [0 200 0 0 1]; v = 0;
elseif nargin == 8,
	opt2 = []; opt3 = [0 200 0 0 1]; v = 0;
elseif nargin == 7,
	opt1 = [0 200 0 0 1]; opt2 = []; opt3 = [0 200 0 0 1]; v = 0;
elseif nargin <= 6,
	error('Incorrect number of arguments');
end;

% Preliminaries

N=Avec(2);

% Dvec
str=[];
for i=1:N
	istr=int2str(i); aux=[',D',istr]; str=[str,aux];
	eval(['D',istr,'=D;']);
end;
str(1)=[]; eval(['Dvec=stack(',str,');']);

%%% 1). Initial solution
if v
	fprintf('\n\n----------------------------\n');
	fprintf(    'Optimal Hinf control Problem\n');
	fprintf(    '----------------------------\n\n');
	fprintf(    '1. Initial condition\n');
	pause(0.2);
end;

[Fvec,nu_0] = hinfopt(Avec,Bvec,D,Mvec,Jvec,P,opt3,0);

if v
	fprintf('   First estimate for nu: %d\n\n',nu_0);
	pause(0.2);
end;

% There is a good chance that this initial solution is near the optimal one
[Fvec,Xvec,infodat] = hinfsopt(Avec,Bvec,Dvec,Mvec,Jvec,P,0.95*nu_0,opt1,opt2);

if infodat(1)<0 | infodat(2)==1 | infodat(3)==1
	up_nu = nu_0; lo_nu = 0.95*nu_0;
else
	up_nu = 0.95*nu_0; lo_nu = 0;
end;

%%% 2). Iteration phase

n_it = ceil(log(tol/(up_nu-lo_nu))/log(0.5));

if v
	fprintf('2. Iteration phase\n');
	fprintf('   Desired tolerance will be achieved in %d steps\n\n',n_it);
	fprintf('--st.#---Dir---LowBound----UppBound----CurValue----Comments------------\n');
	pause(0.2);
end;

it = 1;

while (up_nu - lo_nu) >= tol
	
	nu_cur = lo_nu + (up_nu-lo_nu)/2;

	[Fvec,Xvec,infodat] = hinfsopt(Avec,Bvec,Dvec,Mvec,Jvec,P,nu_cur,opt1,opt2);

	if infodat(1)<0 | infodat(2)==1 | infodat(3)==1
	   % No convergence
	   if v
	      arrow = '>>';
	      if infodat(1)<0
	         diagstr = 'No convergence';
	      elseif infodat(2)==1 & infodat(3)==1
	         diagstr = 'Conditions (i) and (ii) failed';
	      elseif infodat(2)==1
	         diagstr = 'Condition (i) failed';
	      elseif infodat(3)==1
	         diagstr = 'Condition (ii) failed';
	      end;
	      fprintf('%6.0f\t %s    %1.3e  %1.3e  %1.3e  %s\n',it,arrow,lo_nu,up_nu,nu_cur,diagstr);
	      pause(0.2);
	   end;
	   lo_nu = nu_cur;
	else
	   % Convergence
	   if v
	      arrow = '<<';
	      diagstr = sprintf('Convergence in %d iterations',infodat(1));
	      fprintf('%6.0f\t %s    %1.3e  %1.3e  %1.3e  %s\n',it,arrow,lo_nu,up_nu,nu_cur,diagstr);
	      pause(0.2);
	   end;
	   up_nu = nu_cur;
	end;

	it = it+1;
end;	

%%% 3). Conclusion

nuopt = up_nu;